# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/06_seed.ipynb.

# %% auto 0
__all__ = ['seedall']

# %% ../nbs/06_seed.ipynb 6
from functools import wraps

# %% ../nbs/06_seed.ipynb 8
from types import FunctionType

# %% ../nbs/06_seed.ipynb 11
#| export

# %% ../nbs/06_seed.ipynb 13
#| export


# %% ../nbs/06_seed.ipynb 15
#| export


# %% ../nbs/06_seed.ipynb 17
#| export


# %% ../nbs/06_seed.ipynb 19
from lull import lull
from seeder import seedall as _seedall

# %% ../nbs/06_seed.ipynb 21
#| export


# %% ../nbs/06_seed.ipynb 23
@wraps(_seedall)
def seedall(seed: int = 3, reseed: bool = False) -> int:
    '''Set seeds for reproducibility.
    
    Parameters
    ----------
    seed : int
    
    reseed : bool, default = False,
        Whether or not to reinvoke each seed function if it was already called.

    Returns
    -------
    seed: int
        The seed used to set the random number generators.

    Notes
    -----
    There is a lot to say about [randomness](https://pytorch.org/docs/stable/notes/randomness.html) and 
    reproducibiliity in python. For a more detailed discussion, see the 
    documentation on 
    [random.seed](https://docs.python.org/3/library/random.html#random.seed), 
    [np.random.seed](https://numpy.org/doc/stable/reference/random/generated/numpy.random.seed.html),
    [torch.manual_seed](https://pytorch.org/docs/stable/generated/torch.manual_seed.html), and
    [pl.seed_everything]( https://pytorch-lightning.readthedocs.io/en/latest/common/seed_everything.html).

    See Also
    --------
    random.seed, np.random.seed, torch.manual_seed, pl.seed_everything

    .. _randomness: https://pytorch.org/docs/stable/notes/randomness.html
    .. _random.seed: https://docs.python.org/3/library/random.html#random.seed
    .. _np.random.seed: https://numpy.org/doc/stable/reference/random/generated/numpy.random.seed.html
    .. _torch.manual_seed: https://pytorch.org/docs/stable/generated/torch.manual_seed.html
    .. _pl.seed_everything: https://pytorch-lightning.readthedocs.io/en/latest/common/seed_everything.html
    '''    
    with lull():
        _seedall(seed, reseed)
    return seed

# %% ../nbs/06_seed.ipynb 25
#| export
